#!/usr/bin/perl -w
use strict;
use POSIX;

my $sched_prefix;  
my $antenna_code;
my $debug = 0;

if ($#ARGV < 1) {
  die "Usage: $0 <experiment name> <station code> [<debug_mode>]\nWhere debug_mode is optional an 0 or 1\n";
} else {
  $sched_prefix = $ARGV[0];  
  $antenna_code = $ARGV[1];
}
if ($#ARGV == 2) {
  $debug = $ARGV[2];
}

# Forces autoflush to console
$| = 1;


#clear the teminal window
system ("clear");
print "##########################################################\n";
print "##########################################################\n";
print "##                                                      ##\n";
print "##  Warning: This script will start a new dynamic       ##\n";
print "##  schedule and stop any schedule currently running!   ##\n";
print "##                                                      ##\n";
print "##########################################################\n";
print "##########################################################\n";
print "\n";
print "Are you sure you want to continue (y/[n])? ";
my $reply;
chomp($reply = <STDIN>);
if (!($reply =~ /[yY]/)) {
  die "OK, stopping here. No changes made to the schedule";
}

print "\n##########################################################\n\n";


#-------------------------------------------------------------------------------
# definitions:
#-------------------------------------------------------------------------------

# Code for this antenna. First character should be a capital
$antenna_code= ucfirst(lc($antenna_code));
# lower-case version of the above
my $antenna_code_lc = uc2lc($antenna_code);



# Where this program lives and where ancilliary files should be placed
my $working_directory="/home/oper/dysched";

# Read this file. If it contains the text "override=true" it will not adjust the schedule, assuming
# someone has locked off the antenna
my $override_file="override.txt";

# get current schedule name
#my $sched_prefix;
#open (LOGNM, "/usr2/fs/bin/lognm |") || die "Failed to execute lognm: $!\n";
#while ( <LOGNM> ) {
#    chomp;
#   $sched_prefix = $_;
#} 
#close LOGNM;
my $active_schedule=$sched_prefix.$antenna_code_lc.".snp";
print "Current active schedule is $active_schedule\n";

# The program to run to return the antenna availability status
my $ant_status_prog="/usr2/st/mondump/mondump";

# Where the skd files are kept on the field system PC
my $sched_file_dir="/usr2/sched";
my $sched_file_name="$sched_file_dir/$active_schedule";

# Where to find the dynamic schedule status information
my $status_file_url_root="http://auscope.phys.utas.edu.au";
my $status_file_name="dynamic_sched_status.html";
my $status_file_url="$status_file_url_root/$status_file_name";

# location of dynamic schedule SKD files
my $dysched_url = "http://fourier.phys.utas.edu.au/DS";


#-------------------------------------------------------------------------------
my ($antennas, $latest_sched_file, $sched_timestamp, $ts_year, $ts_day, $ts_timestr, $ts_h, $ts_m);
my ($command_y, $command_d, $command_h, $command_m, $command_s);
my ($latest_sched_time, $latest_command_time, $antenna_available);
my ($dummy, $override_status);
my ($unixtime, $len, $i);
my ($future_time);

# for these purposes, OK to assume 366 days per year
my $ndays = 366;

chdir $working_directory;


# on startup we create a new schedule file based on /usr2/sched/ds.snp and run it
system ("cp /usr2/sched/ds.snp /usr2/sched/$active_schedule");
system ("cp /usr2/proc/ds.prc /usr2/proc/$sched_prefix$antenna_code_lc.prc");

# Calculate how many lines in the template file
open(SNPFILE, "/usr2/sched/$active_schedule") || die "Can't open $active_schedule: $!\n";
my $index;
$index++ while<SNPFILE>;
close SNPFILE;
#print "line = $index\n";


open(SNPFILE, ">>/usr2/sched/$active_schedule") || die "Can't open $active_schedule: $!\n";
#need to give a timestamp so that main loop knows when to activate. 
$unixtime =  strftime("%Y.%j.%H:%M:%S", gmtime(time+70));
print SNPFILE "!$unixtime\n";
#print SNPFILE "!+1m\n";
# 
my $text = sprintf "schedule=$sched_prefix$antenna_code_lc,#"."%d\n",$index+3;
#print SNPFILE "schedule=$sched_prefix$antenna_code_lc,#14\n";
print SNPFILE $text;
system("inject_snap schedule=$sched_prefix$antenna_code_lc,#1");

# main loop
while (1) {
  chdir $working_directory;
  print "Checking antenna availability...\n";
  ###################################################################################
  # Is the antenna available?
  if ($debug) {
    print "Checking antenna availability with $ant_status_prog\n";
  } 


  ($antenna_available) = get_antenna_availability($ant_status_prog);
  if ($debug) {
      print "(after get_antenna_availability) antenna_available = $antenna_available\n";
  }
  ###################################################################################
  if ($debug) {
    print "Checking antenna availability in $override_file\n";
  }
  open(STATFILE, $override_file) || die "Can't open $override_file: $!\n";
  while (<STATFILE>) {
    chomp;
    if (/^override=/) {
      ($dummy, $override_status) = split /=/ ; #/
    }
  }
  close STATFILE;
  $override_status=uc2lc($override_status);
  if ($debug) {
    print "override_status = $override_status\n";
  }
  if ($override_status =~ "true") {
    if ($debug) {print "Antenna has been locked out\n";}
    $antenna_available = 0;
  } else {
    if ($debug) {
      print "Antenna has NOT been locked out\n";
      print "antenna_available = $antenna_available\n";
    }
  }

  if ($antenna_available) {
    print "Antenna is available for use\n";
  } else {
    print "Antenna is NOT available for use\n";
  }

  print "----------------------------------------------------------\n";

  ###################################################################################
  if ($antenna_available) {
    if ($debug) {
      print "Fetching latest dynamic scheduling status file $status_file_url\n";
    } else {
      print "Checking for an addition to the schedule...\n";
    }
    system("rm $status_file_name");
    if ($debug) {
      system ("wget --no-cache $status_file_url");
    } else {
      system ("wget --no-cache -q $status_file_url");
    }
    open(STATFILE, $status_file_name) || die "Can't open $status_file_name: $!\n";
    while (<STATFILE>) {
      if (/$antenna_code/) {
	($antennas, $latest_sched_file, $sched_timestamp) = split;
      }
    }
    close STATFILE;
    if ($debug) {
      print "Status file says:\n";
      print "  Antennas = $antennas, ";
      print "Latest available sched file = $latest_sched_file, ";
    }
    ($ts_year, $ts_day, $ts_timestr) = split /\./, $sched_timestamp; #/
      $ts_day = sprintf ("%d", $ts_day);
    $ts_h = sprintf ("%d",substr $ts_timestr, 0, 2);
    $ts_m = sprintf ("%d",substr $ts_timestr, 2, 2);
    if ($debug) {
      print "\nStart timestamp = $sched_timestamp ";
      print "(y=$ts_year, d=$ts_day, h=$ts_h, m=$ts_m)\n";
    }
    # Adding 59 seconds to the sched time as Lucia's scheduler takes previous scans into consideration, 
    # and uses a floor for the start minute. 
    $latest_sched_time = $ts_year + ($ts_day/$ndays) + ($ts_h/24/$ndays) + ($ts_m/60/24/$ndays) +(59/3600/24/$ndays);
    if ($debug) {
      print "or, in years $latest_sched_time\n";
      print "----------------------------------------------------------\n\n";
    }
  }
  ###################################################################################
  if ($debug) {
    print "Now reading the active schedule file $sched_file_name to see when the last command happens\n";
  } 
    
  open(SCHEDFILE, $sched_file_name) || die "Can't open $sched_file_name: $!\n";
  while (<SCHEDFILE>) {
    if (/^!/) {
      ($command_y, $command_d, $command_h, $command_m, $command_s) = split /\.|\:/, substr($_,1); #/
    }
  }
  $command_d  = sprintf ("%d", $command_d);
  $command_h  = sprintf ("%d", $command_h);
  $command_m  = sprintf ("%d", $command_m);
  $command_s  = sprintf ("%d", $command_s);
  if ($debug) {
    print "Current last scheduled time = $command_y, $command_d, $command_h, $command_m, $command_s\n";
  }
  $latest_command_time = $command_y + ($command_d/$ndays) + ($command_h/24/$ndays) 
    + ($command_m/60/24/$ndays) + ($command_s/60/60/24/$ndays);
  if ($debug) {
    print "...or in years $latest_command_time\n";
  }
    
  ###################################################################################
  if ($antenna_available) { 
    if ($latest_command_time < $latest_sched_time) {
      if ($debug) {
	print "The latest sched file is in the future: get it!\n";
      } else {
	print "There's an addition to the schedule available. Fetching it...\n";
      }
      chdir $sched_file_dir;
      print "rm $latest_sched_file new.skd\n";
      system ("rm $latest_sched_file new.skd");
      if ($debug) {
	system ("wget $dysched_url/$latest_sched_file");
      } else {
	system ("wget -q $dysched_url/$latest_sched_file");
      }
      print "cp $latest_sched_file new.skd\n";
      system ("cp $latest_sched_file new.skd");
      if ($debug) {
	print "Now drudge and append\n";
      } else {
	print "New addition to the schedule has been retrieved. Appending it to the current one...\n";
      }
      ########## Next line needs testing LIVE
      system ("$working_directory/DynamicSnap.sh $sched_prefix $antenna_code_lc");
    } else {
      if ($debug) {
	print "The latest sched file is in the past: ignore it\n";
      } else {
	print "No new schedule addition is currently available.\n";
      }
      # Need to add a scheduled 60 second pause & then go to the correct line number 
#      if ( strftime("%Y",gmtime(time+60))+strftime("%j",gmtime(time+60))/$ndays + strftime("%H",gmtime(time+60))/$ndays/24+strftime("%M",gmtime(time+60))/$ndays/1440 + strftime("%S",gmtime(time+60))/$ndays/86400 > $latest_command_time  ) {
#	  $latest_command_time=$latest_command_time+60/86400/$ndays;
#          $command_y=floor($latest_command_time);
#	  $command_d=floor($ndays*($latest_command_time-$command_y));
#          $command_h=floor($ndays * 24 *($latest_command_time-$command_y -$command_d/$ndays));
#	  $command_m=floor($ndays *1440 * ( $latest_command_time-$command_y -$command_d/$ndays-$command_h/$ndays/24));
#	  $command_s=floor($ndays *86400 * ( $latest_command_time-$command_y -$command_d/$ndays-$command_h/$ndays/24-$command_m/$ndays/1440));
#	  $unixtime = sprintf("!%04d.%03d.%02d:%02d:%02d",$command_y, $command_d, $command_h, $command_m, $command_s);
#          open(SNPFILE, ">>/usr2/sched/$active_schedule") || die "Can't open $active_schedule: $!\n";
#          print SNPFILE "$unixtime\n";
#          $len  = `cat /usr2/sched/$active_schedule | wc -l ` + 2 ;
#			   print SNPFILE "schedule=$sched_prefix$antenna_code_lc,#$len\n" ; 
    }
  } else {
    print "Antenna is not available, will not adjust the schedule\n";
  }
# Need to add a loop checking to see if the schedule is about to terminate. If so, send a halt. Else send cont 
  $future_time = strftime("%Y",gmtime(time+60)) +
    strftime("%j",gmtime(time+60))/$ndays + 
    strftime("%H",gmtime(time+60))/$ndays/24 +
    strftime("%M",gmtime(time+60))/$ndays/1440 + 
    strftime("%S",gmtime(time+60))/$ndays/86400;
  if ($debug) {print "FT = $future_time\n";}
  if ($future_time > $latest_command_time) {
    system("inject_snap halt");
  }
  else {
    system("inject_snap cont");
  }

  print "Now waiting for 1 minute before checking again...\n";
  for ($i = 0; $i < 6; $i++) {
    sleep 10;
    print "*";
  }
  print "\n----------------------------------------------------------\n\n";
}

sub get_antenna_availability {
  my ($executable) = @_;
  my ($dummy,$status,$ok);
  open (ANTSTAT, "$executable |") || die "Failed to execute $executable: $!\n";
  while ( <ANTSTAT> ) {
    chomp;
    if (/<antenna id=\"available\">/) {
      if ($debug) {print "line found : $_\n";}
      ($dummy,$dummy,$status) = split /\>|\</;
      if ($debug) {print "status = $status\n";}
    }
  } 
  close ANTSTAT;
  $status = uc2lc($status);
  if ($status =~ "yes")  {
    $ok = 1;
  } else {
    $ok = 0;
  }

  # force availability to 1 for now
  $ok = 1;
  return ($ok);
}

sub uc2lc {
# converts upper case to lower case
# call $lc_string = uc2lc($string);
    my ($lc_string) = @_;
    $lc_string = "\L$_[0]\E";
    return $lc_string;
}
